<?php

   if(!defined('GEN_START')) {
      http_response_code(404);
      die();
   }

   ob_start();
   require_once DIR_SYS . 'core' . DIRECTORY_SEPARATOR . 'log.php';
   include_once DIR_SYS . 'external' . DIRECTORY_SEPARATOR . 'kint' . DIRECTORY_SEPARATOR . '_main.php';
   require_once DIR_SYS . 'core' . DIRECTORY_SEPARATOR . 'handler.php';

   spl_autoload_register('\Alo\Handler::autoloader');
   set_error_handler('\Alo\Handler::error', ini_get('error_reporting'));
   set_exception_handler('\Alo\Handler::ecxeption');

   /**
    * A shortcut to isset($var) ? $var : null
    *
    * @author Art <a.molcanovas@gmail.com>
    *
    * @param mixed $var The variable to "return"
    *
    * @return mixed
    */
   function get(&$var) {
      return isset($var) ? $var : null;
   }

   /**
    * Returns a debug string of the passed on variables
    *
    * @return string
    */
   function debug() {
      if(!Kint::enabled()) {
         return null;
      } else {
         ob_start();
         $args = func_get_args();
         call_user_func_array(['Kint', 'dump'], $args);

         return ob_get_clean();
      }
   }

   /**
    * Check if the server is running Windows
    *
    * @author Art <a.molcanovas@gmail.com>
    * @return bool
    */
   function server_is_windows() {
      return substr(strtoupper(php_uname('s')), 0, 3) === 'WIN';
   }

   /**
    * Returns a lite debug string of passed on variables
    *
    * @return string
    */
   function lite_debug() {
      if(!Kint::enabled()) {
         return '';
      } else {
         ob_start();
         $argv = func_get_args();
         echo '<pre>';
         foreach($argv as $k => $v) {
            $k && print("\n\n");
            echo kintLite($v);
         }
         echo '</pre>' . "\n";

         return ob_get_clean();
      }
   }

   /**
    * Returns a very precise timestamp
    *
    * @author Art <a.molcanovas@gmail.com>
    *
    * @param float $microtime Optionally, supply your own microtime
    *
    * @return string Y-m-d H:i:s:{milliseconds}
    */
   function timestamp_precise($microtime = null) {
      if(!$microtime) {
         $microtime = microtime(true);
      }
      $t = explode('.', $microtime);

      return date('Y-m-d H:i:s', $t[0]) . ':' . round($t[1] / 10);
   }

   if(!function_exists('getallheaders')) {

      /**
       * Implement getallheaders() for non-apache servers
       *
       * @author Art <a.molcanovas@gmail.com>
       * @return array
       */
      function getallheaders() {
         $headers = [];
         foreach($_SERVER as $name => $value) {
            if(substr($name, 0, 5) == 'HTTP_') {
               $headers[str_replace(' ', '-', ucwords(strtolower(str_replace('_', ' ', substr($name, 5)))))] = $value;
            }
         }

         return $headers;
      }
   }

   /**
    * Escapes sensitive characters for HTML5 output
    *
    * @author Art <a.molcanovas@gmail.com>
    *
    * @param string $str The input string
    *
    * @return string
    */
   function escape_html5($str) {
      return htmlspecialchars($str, ENT_QUOTES | ENT_HTML5);
   }

   /**
    * Performs include() only if a file exists
    *
    * @param string $path Path to the file
    *
    * @return bool true if the file exists, false if not.
    */
   function includeifexists($path) {
      if(file_exists($path)) {
         include $path;

         return true;
      }

      return false;
   }

   /**
    * Performs include_once() only if a file exists
    *
    * @param string $path Path to the file
    *
    * @return bool true if the file exists, false if not.
    */
   function includeonceifexists($path) {
      if(file_exists($path)) {
         include_once $path;

         return true;
      }

      return false;
   }

   require_once DIR_SYS . 'core' . DIRECTORY_SEPARATOR . 'alo.php';

   includeonceifexists(DIR_APP . 'core' . DIRECTORY_SEPARATOR . 'autoload.php');

   if(!defined('PHPUNIT_RUNNING')) {
      Alo::$router = new Alo\Controller\Router();
      Alo::$router->init();
   }